-- Self Whisper Test Purposes
-- SendChatMessage(("Message" .. var), "WHISPER", nil, "Tex")

local AzerPUG_RareUtility, RareUtil = ...

local L = LibStub("AceLocale-3.0"):GetLocale("AzerPUGRareUtility", false)

local lastTargetID = nil
local UpdateInterval = 1.0
local UpdateSecondCounter = 0
local raresInfo = RareUtil.raresInfo
local raresRotation = RareUtil.raresRotation
local addonVersion = "v2.1"
local zone = GetZoneText()
local zoneID = C_Map.GetBestMapForUnit("player")
local announceChannel = nil
local zoneShardID = nil
local addonChannelName = "RareUtilAC"

local addon = LibStub("AceAddon-3.0"):NewAddon("RareUtil", "AceConsole-3.0")
local RareUtilLDB = LibStub("LibDataBroker-1.1"):NewDataObject("RareUtil", {
	type = "data source",
	text = "RareUtil",
	icon = "Interface\\Icons\\Inv_darkmoon_eye",
	OnClick = function() addon:ShowHideFrame() end
})
local icon = LibStub("LibDBIcon-1.0")

local dash = " - "
local promo = "AzerPUG RareUtility (8.3 BetaBuild)"

function addon:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("RareUtilDB", {
		profile = {
			minimap = {
				hide = false,
			},
		},
	})
	icon:Register("RareUtil", RareUtilLDB, self.db.profile.minimap)
	self:RegisterChatCommand("RareUtil icon", "MiniMapIconToggle")
end

function addon:MiniMapIconToggle()
	self.db.profile.minimap.hide = not self.db.profile.minimap.hide
	if self.db.profile.minimap.hide then
		icon:Hide("RareUtil")
	else
		icon:Show("RareUtil")
	end
end

function addon:OnLoad(self)
    local RareUtilAddonFrame = CreateFrame("FRAME", "RareUtilAddonFrame", UIParent)
    RareUtilAddonFrame:SetPoint("CENTER", 0, 0)
    RareUtilAddonFrame.texture = RareUtilAddonFrame:CreateTexture()
    RareUtilAddonFrame.texture:SetAllPoints(true)
    RareUtilAddonFrame:EnableMouse(true)
    RareUtilAddonFrame:SetMovable(true)
    RareUtilAddonFrame:SetScript("OnUpdate", function(...) addon:OnUpdate(...) end)
    RareUtilAddonFrame:SetScript("OnEvent", function(...) addon:OnEvent(...) end)
    RareUtilAddonFrame:RegisterForDrag("LeftButton")
    RareUtilAddonFrame:SetScript("OnDragStart", RareUtilAddonFrame.StartMoving)
    RareUtilAddonFrame:SetScript("OnDragStop", RareUtilAddonFrame.StopMovingOrSizing)
    RareUtilAddonFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
    RareUtilAddonFrame:RegisterEvent("ZONE_CHANGED_NEW_AREA")
    RareUtilAddonFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
    RareUtilAddonFrame.TimeSinceLastUpdate = 0
    RareUtilAddonFrame.MinuteCounter = 0

    local AddonTitle = RareUtilAddonFrame:CreateFontString("AddonTitle", "ARTWORK", "GameFontNormalLarge")
    AddonTitle:SetText("AzerPUG RareUtility")
    --AddonTitle:SetWidth("100")
    AddonTitle:SetHeight("10")
    AddonTitle:SetPoint("TOP", "RareUtilAddonFrame", -75, -15)

    local TempFrameSplitter = CreateFrame("FRAME", "TempFrameSplitter", RareUtilAddonFrame)
    TempFrameSplitter:SetPoint("TOP", 0, -40)
    TempFrameSplitter.texture = TempFrameSplitter:CreateTexture()
    TempFrameSplitter.texture:SetAllPoints(true)

    local OptionsPanel = CreateFrame("FRAME", "AzerPUG RareUtility")
    OptionsPanel.name = "AzerPUG RareUtility"
    InterfaceOptions_AddCategory(OptionsPanel)

    local OptionsHeader = OptionsPanel:CreateFontString("OptionsHeader", "ARTWORK", "GameFontNormalHuge")
    OptionsHeader:SetText(promo .. dash .. L["Options"])
    OptionsHeader:SetWidth(OptionsPanel:GetWidth())
    OptionsHeader:SetHeight(OptionsPanel:GetHeight())
    OptionsHeader:SetPoint("TOP", 0, -10)

    local OptionsMessage = OptionsPanel:CreateFontString("OptionsMessage", "ARTWORK", "GameFontNormal")
    OptionsMessage:SetText("Due to recent API changes the options panel has been broken.")
    OptionsMessage:SetWidth(OptionsPanel:GetWidth())
    OptionsMessage:SetHeight(OptionsPanel:GetHeight())
    OptionsMessage:SetPoint("TOP", 0, -75)

    local OptionsMessage = OptionsPanel:CreateFontString("OptionsMessage", "ARTWORK", "GameFontNormal")
    OptionsMessage:SetText("We are working on a fix, but for now it had to be temorarily disabled!")
    OptionsMessage:SetWidth(OptionsPanel:GetWidth())
    OptionsMessage:SetHeight(OptionsPanel:GetHeight())
    OptionsMessage:SetPoint("TOP", 0, -90)

    local RareUtilityAnnounceButton = CreateFrame("Button", "RareUtilityAnnounceButton", RareUtilAddonFrame, "UIPanelButtonTemplate")
    RareUtilityAnnounceButton.contentText = RareUtilityAnnounceButton:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
    RareUtilityAnnounceButton.contentText:SetText("Announce Rare")
    RareUtilityAnnounceButton:SetWidth("100")
    RareUtilityAnnounceButton:SetHeight("25")
    RareUtilityAnnounceButton.contentText:SetWidth("100")
    RareUtilityAnnounceButton.contentText:SetHeight("15")
    RareUtilityAnnounceButton:SetPoint("TOP", 75, -5)
    RareUtilityAnnounceButton.contentText:SetPoint("CENTER", 0, -1)
    RareUtilityAnnounceButton:SetScript("OnClick", function() addon:CheckRareMob(true) end )
end

function addon:OnFrameShow()
    zone = GetZoneText()
    zoneID = C_Map.GetBestMapForUnit("player")
    if raresInfo[zone] ~= nil then
        local frameHeight = (((#raresInfo[zoneID] / 2) * 12) + 45)
        --if zoneID == 1530 then      --Vale of Eternal Blossoms
        --    frameHeight = nil
        --elseif zoneID == 1524 then   --Uldum
        --    frameHeight = nil
        --else
            --frameHeight = (((#raresInfo[zoneID] / 2) * 12) + 45)
        --end
        RareUtilAddonFrame:SetSize(300, frameHeight)
        TempFrameSplitter:SetSize(5, frameHeight)
    end
    addon:ShowZoneFrame(zoneID)
end

function addon:ShowZoneFrame(ShowZone)
    local rareAmount = nil
    local rotNumb = nil
    addon:HideAllRares()
    if ShowZone == 1527 or ShowZone == 1530 then
        if ShowZone == 1527 then
            if C_AreaPoiInfo.GetAreaPOISecondsLeft(6486) ~= nil then
                rotNumb = 6486
            elseif C_AreaPoiInfo.GetAreaPOISecondsLeft(6487) ~= nil then
                rotNumb = 6487
            elseif C_AreaPoiInfo.GetAreaPOISecondsLeft(6488) ~= nil then
                rotNumb = 6488
            end
        elseif ShowZone == 1530 then
            if C_AreaPoiInfo.GetAreaPOISecondsLeft(6489) ~= nil then
                rotNumb = 6489
            elseif C_AreaPoiInfo.GetAreaPOISecondsLeft(6490) ~= nil then
                rotNumb = 6490
            elseif C_AreaPoiInfo.GetAreaPOISecondsLeft(6491) ~= nil then
                rotNumb = 6491
            end
        end
        rareAmount = #raresRotation[rotNumb]
        for i = 1, #raresInfo[ShowZone] do
            for j = 1, rareAmount do
                local raresZI = raresInfo[ShowZone][i]
                if raresZI[1] == raresRotation[rotNumb][j] then
                    raresZI[6]:Show()
                    raresZI[6].contentText:SetTextColor(addon:ButtonColors(ShowZone, raresZI[1], raresZI[9]))
                    if j <= math.ceil(rareAmount / 2) then
                        raresZI[6]:SetPoint("TOP", -125, (-12 * j - 30))
                        raresZI[6].contentText:SetPoint("LEFT", 0, 0)
                    else
                        raresZI[6]:SetPoint("TOP", 125, (-12 * (j - math.ceil(rareAmount / 2)) -30))
                        raresZI[6].contentText:SetPoint("RIGHT", 0, 0)
                    end
                end
            end
        end
        RareUtilAddonFrame:Show()
        local frameHeight = (((#raresRotation[rotNumb] / 2) * 12) + 50)
        RareUtilAddonFrame:SetSize(400, frameHeight)
        TempFrameSplitter:SetSize(3, (frameHeight - 45))
    elseif ShowZone == 1355 or ShowZone == 1462 then
        rareAmount = #raresInfo[ShowZone]
        for i = 1, rareAmount do
            local raresZI = raresInfo[ShowZone][i]
            raresZI[6]:Show()
            raresZI[6].contentText:SetTextColor(addon:ButtonColors(ShowZone, raresZI[1], raresZI[9]))
            if i <= math.ceil(rareAmount / 2) then
                raresZI[6]:SetPoint("TOP", -100, (-12 * i - 30))
                raresZI[6].contentText:SetPoint("LEFT", 0, 0)
            else
                raresZI[6]:SetPoint("TOP", 100, (-12 * (i-math.ceil(rareAmount / 2)) -30))
                raresZI[6].contentText:SetPoint("RIGHT", 0, 0)
            end
        end
        RareUtilAddonFrame:Show()
        local frameHeight = (((#raresInfo[ShowZone] / 2) * 12) + 45)
        RareUtilAddonFrame:SetSize(400, frameHeight)
        TempFrameSplitter:SetSize(3, (frameHeight - 45))
    end
        --raresZI[7]:Show()
        --raresZI[7]:SetTexture("Interface/AddOns/AzerPUG RareUtility/CrossIcon.tga", true)
        --local rareKillIcon = raresZI[7]:CreateTexture(nil, "BACKGROUND")
        --rareKillIcon:SetTexture(addon:KillMarkColors(iterationZone, raresZI[1], raresZI[9]))
        --rareKillIcon:SetAllPoints()
        --raresZI[7]:SetColorTexture(addon:KillMarkColors(iterationZone, raresZI[1], raresZI[9]))

    

    -- local frameHeight = (((#raresInfo[ShownZone] / 2) * 12) + 45)
    -- RareUtilAddonFrame:SetSize(400, frameHeight)
    -- TempFrameSplitter:SetSize(3, (frameHeight - 45))

    --raresInfo[1355][1][6]:SetPoint("TOP", -100, 500)      Change button position, no need to recreate!
end

function addon:HideAllRares()
    for iterationZone, rares in pairs(raresInfo) do
        local rareAmount = #raresInfo[iterationZone]
        for i = 1, rareAmount do
            rares[i][6]:Hide()
            --rares[i][7]:Hide()
        end
    end
end

function addon:ShowHideFrame()
    if RareUtilAddonFrame:IsShown() then
        RareUtilAddonFrame:Hide()
    elseif not RareUtilAddonFrame:IsShown() then
        RareUtilAddonFrame:Show()
        if raresInfo[zoneID] ~= nil then
            addon:ShowZoneFrame(zoneID)
        end
    end
end

function addon:OnUpdate(self, elapsed)
    self.TimeSinceLastUpdate = self.TimeSinceLastUpdate + elapsed
    while (self.TimeSinceLastUpdate > UpdateInterval) do
        self.TimeSinceLastUpdate = self.TimeSinceLastUpdate - UpdateInterval
        UpdateSecondCounter = UpdateSecondCounter + UpdateInterval
    end
    if UpdateSecondCounter >= 60 then
        UpdateSecondCounter = UpdateSecondCounter - 60
    end
end

function addon:CreateUnitButtons()
    for iterationZone, rares in pairs(raresInfo) do
        local rareAmount = #raresInfo[iterationZone]
        for i = 1, rareAmount do
            local raresZI = rares[i]
            if raresZI[6] == nil then
                raresZI[6] = CreateFrame("Button", nil, RareUtilAddonFrame)
                raresZI[6].contentText = raresZI[6]:CreateFontString(nil, "ARTWORK", "GameFontNormal")
                raresZI[6].contentText:SetText(raresZI[2])
                raresZI[6]:SetWidth("104")
                raresZI[6]:SetHeight("10")
                --if i <= math.ceil(rareAmount / 2) then
                --    raresZI[6]:SetPoint("TOP", -100, (-12 * i - 30))
                --    raresZI[6].contentText:SetPoint("LEFT", 0, 0)
                --else
                --    raresZI[6]:SetPoint("TOP", 100, (-12 * (i-math.ceil(rareAmount / 2)) -30))
                --    raresZI[6].contentText:SetPoint("RIGHT", 0, 0)
                --end
                local xLocation = raresZI[12][1][1]
                local yLocation = raresZI[12][1][2]
                local name = raresZI[2]
                raresZI[6]:SetScript("OnClick",
                function()
                    if xLocation == "??" or yLocation == "??" then
                        print(L["Rare has multiple spawns!"])
                    else
                        TomTom:AddWaypointToCurrentZone(xLocation, yLocation, name) 
                    end
                end)
            end
        end
    end
end

--[[
function addon:CreateUnitKillMarkers()
    for iterationZone, rares in pairs(raresInfo) do
        local rareAmount = #raresInfo[iterationZone]
        for i = 1, rareAmount do
            local raresZI = rares[i]
            if raresZI[7] == nil then

                raresZI[7] = CreateFrame("FRAME", nill, RareUtilAddonFrame)
                local rareKillIcon = raresZI[7]:CreateTexture(nil, "BACKGROUND")
                --rareKillIcon:SetTexture("CrossIcon.tga", true)
                rareKillIcon:SetTexture("Interface/AddOns/AzerPUG RareUtility/CrossIcon.tga", true)
                --raresZI[7]:SetTexture("CrossIcon.tga", true)
                rareKillIcon:SetAllPoints()
                --rareKillIcon:SetColorTexture(0, 1, 1, 1)

                raresZI[7]:SetWidth("6")
                raresZI[7]:SetHeight("6")
                if i <= math.ceil(rareAmount / 2) then
                    raresZI[7]:SetPoint("TOP", -135, (-12 * i - 34))
                    --raresZI[7].contentText:SetPoint("RIGHT", 0, 0)
                else
                    raresZI[7]:SetPoint("TOP", 135, (-12 * (i - math.ceil(rareAmount / 2)) - 34))
                    --raresZI[7].contentText:SetPoint("LEFT", 0, 0)
                end
            end
        end
    end
end
]]

function addon:SavedVarsLoad()
    RareUtilAddonFrame.texture:SetColorTexture(0.5, 0.5, 0.5, 0.5)
    TempFrameSplitter.texture:SetColorTexture(0, 0, 0, 1)
end

function addon:OnZoning()
    LeaveChannelByName(addonChannelName)
    zone = GetZoneText()
    zoneID = C_Map.GetBestMapForUnit("player")
    if raresInfo[zoneID] ~= nil then
        JoinTemporaryChannel("ChannelIDSkipp")
        JoinTemporaryChannel(addonChannelName)
        addon:ShowZoneFrame(zoneID)
        addon:DelayedExecution(5, function() LeaveChannelByName("ChannelIDSkipp") end)
    end
end

function addon:CheckAddonChannelID()
    local addonChannelID = GetChannelName(addonChannelName)
    return addonChannelID
end

function addon:SendAddonMessage(UnitInputID, ChatOrAddon)
    local addonChannelID = GetChannelName(addonChannelName)
    if zoneShardID ~= nil then
        SendChatMessage(("RareUtil-" .. tostring(zoneShardID) .. "-" .. tostring(UnitInputID) .. "-" .. tostring(ChatOrAddon)), "CHANNEL", nil, addonChannelID)
    end
end

function addon:NameToID(rareName)
    if raresInfo[zoneID] ~= nil then
        for i = 1, #raresInfo[zoneID] do
            local rareZI = raresInfo[zoneID][i]
            if rareZI[2] == rareName then
                return rareZI[1]
            end
        end
    end
end

function addon:IDToName(rareID)
    if raresInfo[zoneID] ~= nil then
        for i = 1, #raresInfo[zoneID] do
            local rareZI = raresInfo[zoneID][i]
            if rareZI[1] == rareID then
                return rareZI[2]
            end
        end
    end
end

function addon:EnterWorldChecks(counterX)
    zone = GetZoneText()
    zoneID = C_Map.GetBestMapForUnit("player")
    addon:ShowZoneFrame(zoneID)
    announceChannel = addon:CheckGeneralChat()
end

function addon:OnEvent(self, event, ...)
    if event == "PLAYER_ENTERING_WORLD" then
        addon:CreateUnitButtons()
        --addon:CreateUnitKillMarkers()
        addon:SavedVarsLoad()

        RareUtilAddonFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
        RareUtilAddonFrame:RegisterEvent("CHAT_MSG_CHANNEL")
        RareUtilAddonFrame:RegisterEvent("CHAT_MSG_WHISPER")

        local counterX = 0
        counterX = addon:EnterWorldChecks(counterX)
        while GetZoneText() == nil do
            counterX = addon:EnterWorldChecks(counterX)
        end
    elseif event == "ZONE_CHANGED_NEW_AREA" then
        addon:OnZoning()
    elseif event == "CHAT_MSG_CHANNEL" then
        local pName, pServer = UnitFullName("player")
        local playerFullName = (pName .. "-" .. pServer)
        local msgText, msgSender, _, msgChannelName = ...
        local locDeath = L[" dead!"]
        local rareName, rareNamep2, rareHealth = nil
        local rareID = nil
        if msgChannelName == addonChannelName then
            addon:GetRareTimeInfo(msgText)
        end

        if string.find(msgText, promo) and string.find(msgText, "%%") and not string.find(msgText, locDeath) then
            local locEnforcer = L["Enforcer KX-T57"]
            local locAvenger = L["OOX-Avenger/MG"]
            local locCruncher = L["Gorged Gear-Cruncher"]
            local locHaunter = L["Chasm-Haunter"]
            if string.find(msgText, locEnforcer) or string.find(msgText, locAvenger) or string.find(msgText, locCruncher) or string.find(msgText, locHaunter) then
                rareName, rareNamep2, _, rareHealth = strsplit("-", msgText)
                rareName = (rareName .. "-" .. rareNamep2)
            else
                rareName, _, rareHealth = strsplit("-", msgText)
            end
            rareName = strsub(rareName, 1, (string.len(rareName)-1))
            if string.find(rareName, "%(") then
                strsub(rareName, 1, (string.find(rareName, "%(") - 1))
            end
            local locUnegaged = L["Unengaged!"]
            if string.find(rareHealth, locUnegaged) then
                rareHealth = 100
            else
                rareHealth = strsub(rareHealth, 2, (string.find(rareHealth, "%%") - 1))
                --pick string length, find the dot, if 3 before dot 100. if 2 before dot, use those.
            end

            for i = 1, #raresInfo[zoneID] do
                local rareZI = raresInfo[zoneID][i]
                if rareName == rareZI[2] and msgSender ~= playerFullName then
                    raresInfo[zoneID][i][4] = time()
                    raresInfo[zoneID][i][5] = rareHealth
                end
            end
        elseif string.find(msgText, promo) and string.find(msgText, locDeath) then
            rareName = strsub(msgText, 1, (string.find(msgText, locDeath) - 1))
            rareID = addon:NameToID(rareName)
        elseif string.find(msgText, " has died") then
            rareName = strsub(msgText, 7, (string.find(msgText, " has died") - 1))
            rareID = addon:NameToID(rareName)
        elseif string.find(msgText, " died") then
            rareName = strsub(msgText, 1, (string.find(msgText, " died") - 1))
            rareID = addon:NameToID(rareName)
        end
    ---------------------------------------------------------------------------------------------------------------------------------------
    ----------  Lead Developer-only commands for RealTime debugging in whisper. --------------------------------------------------------------------------
    ---------------------------------------------------------------------------------------------------------------------------------------
    elseif event == "CHAT_MSG_WHISPER" then
        local msgText, msgSender, _, msgChannelName = ...
        if msgSender == "Tex-Ravencrest" and string.find(msgText, "!RareUtil Version") then
            SendChatMessage(("Hey Tex :)     My version is: " .. addonVersion), "WHISPER", nil, "Tex")
        end
        if msgSender == "Tex-Ravencrest" and string.find(msgText, "!RareUtil Shard") then
            SendChatMessage(("Hey Tex :)     My sahrdID is: " .. zoneShardID), "WHISPER", nil, "Tex")
        end
    ---------------------------------------------------------------------------------------------------------------------------------------
    ---------------------------------------------------------------------------------------------------------------------------------------
    ---------------------------------------------------------------------------------------------------------------------------------------
    elseif event == "COMBAT_LOG_EVENT_UNFILTERED" then
        local _, subEvent, _, _, _, _, _, UnitGUID = CombatLogGetCurrentEventInfo()
        if subEvent == "UNIT_DIED" then
            local UnitID = tonumber(addon:GetUnitIDFromGUID(UnitGUID))
            for iterationZone, rares in pairs(raresInfo) do
                if iterationZone == zoneID then
                    for i = 1, #rares do
                        local raresZI = rares[i]
                        if UnitID == raresZI[1] then
                            addon:DelayedExecution(5, function() raresZI[6].contentText:SetTextColor(addon:ButtonColors(iterationZone, raresZI[1], raresZI[9])) end)
                            raresZI[4] = time()
                        end
                    end
                end
            end
            local rareZ = raresInfo[1462]
            local tempIndex = nil
            if UnitID == 149847 or UnitID == 152569 or UnitID == 152570 then
                tempIndex = 7
                addon:DelayedExecution(5, function() rareZ[tempIndex][6].contentText:SetTextColor(addon:ButtonColors(iterationZone, rareZ[tempIndex][1], rareZ[tempIndex][9])) end)
                rares[tempIndex][4] = time()
            elseif UnitID == 154342 then
                tempIndex = 1
                addon:DelayedExecution(5, function() rareZ[tempIndex][6].contentText:SetTextColor(addon:ButtonColors(iterationZone, rareZ[tempIndex][1], rareZ[tempIndex][9])) end)
                rares[tempIndex][4] = time()
            elseif UnitID == 154968 then
                tempIndex = 2
                addon:DelayedExecution(5, function() rareZ[tempIndex][6].contentText:SetTextColor(addon:ButtonColors(iterationZone, rareZ[tempIndex][1], rareZ[tempIndex][9])) end)
                rares[tempIndex][4] = time()
            end
        end
    end
end

function addon:DelayedExecution(delayTime, delayedFunction)       --A big part of this function was obtained with permission, from another addon!
	local frame = CreateFrame("Frame")
	frame.start_time = GetServerTime()
	frame:SetScript("OnUpdate", 
		function(self)
			if GetServerTime() - self.start_time > delayTime then
				delayedFunction()
				self:SetScript("OnUpdate", nil)
				self:Hide()
			end
		end
	)
	frame:Show()
end

function addon:ButtonColors(selectedZoneInput, UnitID, trackInput)
    local selectedZone = selectedZoneInput or zoneID
    for i = 1, #raresInfo[selectedZone] do
        local rareZI = raresInfo[selectedZone][i]
        local trackID = trackInput or rareZI[9]
        if UnitID == rareZI[1] then
            if trackID ~= nil then
                if IsQuestFlaggedCompleted(trackID) == true then
                    return 0, 1, 0, 1
                elseif IsQuestFlaggedCompleted(trackID) == false then
                    return 1, 0, 0, 1
                end
            else
                return 0, 0, 0, 1
            end
        end
    end
    --[[local selectedZone = selectedZoneInput or zoneID
    local LootKnown = addon:AllLootKnown(selectedZone, UnitID)
    for i = 1, #raresInfo[selectedZone] do
        local rareZI = raresInfo[selectedZone][i]
        if UnitID == rareZI[1] then
            if LootKnown == true then 
                return 1, 1, 1, 1
            elseif LootKnown == false then 
                return 1, 0, 0, 1
            end
        end
    end]]
end

function addon:KillMarkColors(selectedZoneInput, UnitID, trackInput)
    local selectedZone = selectedZoneInput or zoneID
    for i = 1, #raresInfo[selectedZone] do
        local rareZI = raresInfo[selectedZone][i]
        local trackID = trackInput or rareZI[9]
        if UnitID == rareZI[1] then
            if trackID ~= nil then
                if IsQuestFlaggedCompleted(trackID) == true then
                    return 0, 1, 0, 1
                elseif IsQuestFlaggedCompleted(trackID) == false then
                    return 1, 0, 0, 1
                end
            else
                return 0, 0, 0, 1
            end
        end
    end
end

function addon:AllLootKnown(selectedZone, UnitID)
    if addon:IsPetKnown(selectedZone, UnitID) == true and addon:IsMountKnown(selectedZone, UnitID) == true and addon:IsToyKnown(selectedZone, UnitID) == true and addon:IsOtherKnown(selectedZone, UnitID) == true then
        return true
    else
        return false
    end
end

function addon:IsMountKnown(selectedZone, UnitID)
    local raresZ = raresInfo[selectedZone]
    for i = 1, #raresZ do
        local rareZI = raresZ[i]
        if UnitID == rareZI[1] then
            if rareZI[13] ~= nil then
                local allMounts = C_MountJournal.GetMountIDs()
                for j = 1, #allMounts do
                    local currentMountLooped = {C_MountJournal.GetMountInfoByID(allMounts[j])}
                    if currentMountLooped[2] == rareZI[13] then
                        if currentMountLooped[11] == true then
                            return true
                        elseif currentMountLooped[11] == false then
                            return false
                        end
                    end
                end
            else
                return true
            end
        end
    end
end

function addon:IsPetKnown(selectedZone, UnitID)
    local raresZ = raresInfo[selectedZone]
    for i = 1, #raresZ do
        local rareZI = raresZ[i]
        if UnitID == rareZI[1] then
            if rareZI[14] ~= nil then
                for petIndex = 1, 1248 do --WoWHead Amount of Pets == 1248
                    local _, _, owned, _, _, _, _, name, _, _, companionID, tooltip, _, _, _, _, _, _ = C_PetJournal.GetPetInfoByIndex(petIndex)
                    if companionID == rareZI[14] then
                        if owned == true then
                            return true
                        elseif owned == false then
                            return false
                        end
                    end
                end
            else
                return true
            end
        end
    end
end

function addon:IsToyKnown(selectedZone, UnitID)
    local raresZ = raresInfo[selectedZone]
    for i = 1, #raresZ do
        local rareZI = raresZ[i]
        if UnitID == rareZI[1] then
            if rareZI[15] ~= nil then
                if PlayerHasToy(rareZI[15]) == true then
                    return true
                elseif PlayerHasToy(rareZI[15]) == false then
                    return false
                end
            else
                return true
            end
        end
    end
end

function addon:IsOtherKnown(selectedZone, UnitID)
    local raresZ = raresInfo[selectedZone]
    for i = 1, #raresZ do
        local rareZI = raresZ[i]
        if UnitID == rareZI[1] then
            if rareZI[16] ~= nil then
                local rareZI16 = rareZI[16]
                for j = 1, #rareZI16 do
                    local rareZI16J = rareZI[16][j]
                    if IsQuestFlaggedCompleted(rareZI16J) == false then
                        return false
                    end
                end
                return true
            else
                return true
            end
        end
    end
end

function addon:CheckGeneralChat()
    if EnumerateServerChannels() ~= nil then
        local channelName = EnumerateServerChannels()
        local mapName = GetZoneText()
        local channelID = GetChannelName(channelName .. " - " .. mapName)
        return channelID
    end
end

function addon:CheckRareMob(buttonClicked)
    if UnitExists("Target") then
        local name = UnitName("target")
        local coords = addon:GetPlayerCoords()
        local unitHealth = addon:GetTargetHealth()
        local UnitID = addon:GetUnitIDFromGUID(UnitGUID("target"))
        if UnitID == 149847 then
            if raresInfo[1462][7][4] < (time() - 300) then
                name = (name .. L[" (Orange)"])
                addon:AnnounceRare(name, coords, unitHealth)
                raresInfo[1462][7][4] = time()
            end
        elseif UnitID == 152569 then
            if raresInfo[1462][7][4] < (time() - 300) then
                name = (name .. L[" (Green)"])
                addon:AnnounceRare(name, coords, unitHealth)
                raresInfo[1462][7][4] = time()
            end
        elseif UnitID == 152570 then
            if raresInfo[1462][7][4] < (time() - 300) then
                name = (name .. L[" (Blue)"])
                addon:AnnounceRare(name, coords, unitHealth)
                raresInfo[1462][7][4] = time()
            end
        elseif UnitID == 154342 then
            name = (name .. L[" (AltTime)"])
            addon:AnnounceRare(name, coords, unitHealth)
            raresInfo[1462][1][4] = time()
        elseif UnitID == 154968 then
            name = (name .. L[" (AltTime)"])
            addon:AnnounceRare(name, coords, unitHealth)
            raresInfo[1462][2][4] = time()
        elseif UnitID == 154225 then
            name = (name .. L[" (AltTime)"])
        elseif UnitID == 151623 then
            name = (name .. L[" (Mount)"])
            if raresInfo[1462][35][4] < (time() - 300) then
                addon:AnnounceRare(name, coords, unitHealth)
                raresInfo[1462][35][4] = time()
            end
        else
            for i = 1, #raresInfo[zoneID] do
                if UnitID == raresInfo[zoneID][i][1] then
                    if UnitID == 151672 then
                        for j = 1, 40 do
                            local _, _, _, _, _, _, _, _, _, v10 = UnitAura("player", j, nil)
                            if v10 == 296644 then
                                name = (name .. L[" (AltTime)"])
                            end
                        end
                    end
                    if buttonClicked == true then
                        if raresInfo[zoneID][i][4] < (time() - 60) then
                            addon:AnnounceRare(name, coords, unitHealth)
                            raresInfo[zoneID][i][4] = time()
                        end
                    end
                end
            end
        end

        if name == "Murloco" then
            addon:AnnounceMurloco()
        end

        if name == "RAG-TAG" then
            addon:AnnounceArmory(coords)
        end
        lastTargetID = UnitID
        for i = 1, #raresInfo[zoneID] do
            if UnitID == raresInfo[zoneID][i] then
            raresInfo[zoneID][i][5] = unitHealth
            end
        end
    else
        lastTargetID = nil
    end
end

function addon:AnnounceRare(name, coords, unitHealth)
    local percent = "%"
    local life = L[" life."]
    announceChannel = addon:CheckGeneralChat()

    if unitHealth > 5 then
        SendChatMessage((name .. dash .. coords .. dash .. unitHealth .. percent .. life .. dash .. promo), "CHANNEL", nil, announceChannel)
    end
end

function addon:AnnounceArmory(coords)
    announceChannel = addon:CheckGeneralChat()
    SendChatMessage((L["Completed Armory"] .. dash .. coords .. dash .. promo), "CHANNEL", nil, announceChannel)
end

function addon:AnnounceMurloco()
    announceChannel = addon:CheckGeneralChat()
    SendChatMessage((L["Murloco found!"] .. dash .. "46 32" .. dash .. promo), "CHANNEL", nil, announceChannel)
end

function addon:GetPlayerCoords()
    local loc = C_Map.GetBestMapForUnit("player")
    local pos = C_Map.GetPlayerMapPosition(loc, "player")
    local coords = math.ceil(pos.x * 10000)/100 .. " " .. math.ceil(pos.y * 10000)/100

    return coords
end

function addon:GetUnitIDFromGUID(UnitGUID)
    local _, _, _, _, _, v6 = strsplit("-", UnitGUID)
    return tonumber(v6)
end

function addon:GetTargetHealth()
    local currentHealth = UnitHealth("target")
    local maxHealth = UnitHealthMax("target")
    local healthPercentage = math.floor(currentHealth / maxHealth * 100)
    return healthPercentage
end

function addon:PrintHelp()
    ChatFrame1:AddMessage("/rareutil help;" .. L["display this message"])
    ChatFrame1:AddMessage("/rareutil options;" .. L["opens the options panel"])
    ChatFrame1:AddMessage("/rareutil frame;" .. L["toggles the Addon Frame"])
    ChatFrame1:AddMessage("/rareutil icon;" .. L["toggles the minimap button/icon"])
end

function addon:OpenOptions()
    InterfaceOptionsFrame_OpenToCategory("AzerPUG RareUtility")
    InterfaceOptionsFrame_OpenToCategory("AzerPUG RareUtility")
end

function addon:AddonFrameToggle()
    if RareUtilAddonFrame:IsShown() then
        RareUtilAddonFrame:Hide()
    elseif not RareUtilAddonFrame:IsShown() then
        RareUtilAddonFrame:Show()
    end
end

SLASH_RAREUTIL1 = "/rareutil"
SlashCmdList["RAREUTIL"] = function(msg, editbox)
    if string.find(msg, "options") then
        addon:OpenOptions()
    elseif string.find(msg, "frame") then
        addon:AddonFrameToggle()
    elseif string.find(msg, "help") then
        addon:PrintHelp()
    end
end

addon:OnLoad()